#include <Wire.h>
#include <HMC5883L.h>
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27,16,2);  // set the LCD address to 0x27 for a 16 chars and 2 line display

// Anschlüsse:
// GND - GND
// VCC - 5V
// SDA - ANALOG Pin 4
// SCL - ANALOG pin 5

 
HMC5883L compass;

int i;
float x_sum, y_sum, z_sum;
float x,y,z;
float x_offset, y_offset;
float Winkel;



// ====================
// ====== SETUP =======
// ====================

void setup()
   {
    Wire.begin();
    Serial.begin(9600);
    //compass = HMC5883L();
    lcd.init();                      // initialize the lcd

    // Print a message to the LCD.
    lcd.backlight();
    lcd.setCursor(2,0);
    lcd.print("Magnetfeld-");
    lcd.setCursor(4,1);
    lcd.print("Sensor");
    
    delay(3000);
    
    lcd.setCursor(2,0);
    lcd.print("           ");
    lcd.setCursor(4,1);
    lcd.print("      ");
    lcd.setCursor(2,0);
    lcd.print("Einheiten:");
    lcd.setCursor(0,1);
    lcd.print("10-6 T bzw. grad");
    
    delay(4000);
    
    lcd.setCursor(2,0);
    lcd.print("           ");
    lcd.setCursor(0,1);
    lcd.print("                ");
    
    
    
    lcd.setCursor(0,0);
    lcd.print("x:");
    lcd.setCursor(9,0);
    lcd.print("y:");
    lcd.setCursor(0,1);
    lcd.print("z:");
    lcd.setCursor(9,1);
    lcd.print("W:");
        
     
    Serial.println("Setting scale to +/- 1.3Ga");
   
    int error = compass.SetScale(1.3);     // Mögliche Bereiche: 0.88, 1.3, 1.9, 2.5, 4.0, 4.7, 5.6, 8.1 Gauss
   
    if(error != 0)
       Serial.println(compass.GetErrorText(error));
 
    Serial.println("Setting measurement mode to continuous");
   
    error = compass.SetMeasurementMode(Measurement_Continuous);
   
    if(error != 0)
       Serial.println(compass.GetErrorText(error));
 
    Serial.println("x:      y:      z:    Total:     Winkel:");
 
 
    x = 0;
    y = 0;
    z = 0;
    Winkel = 0;
    
    
    // offset-Werte
    // ------------
    
    x_offset = 2.44;
    y_offset = 13.15;
   }



// ============================
// ====== HAUPTSCHLEIFE =======
// ============================
 
void loop()
   {
    MagnetometerRaw raw = compass.ReadRawAxis();
    
    x_sum = 0;
    y_sum = 0;
    z_sum = 0;
    
    
    // Mittelwertsbildung:
    // -------------------
    
    for (i = 1; i < 6; i++)
       {
        x_sum += raw.XAxis;    // Einheit milligauss
        y_sum += raw.YAxis;    // Einheit milligauss
        z_sum += raw.ZAxis;    // Einheit milligauss
       }
    
    x = (x_sum / 50.0) + x_offset;   // Mittelwert Einheit µT
    y = (y_sum / 50.0) + y_offset;   // Mittelwert Einheit µT
    z = (z_sum / 50.0);              // Mittelwert Einheit µT
    
       
    // Winkelberechnung
    // ----------------
        
    Winkel = atan2(-x,y);
        
    if(Winkel < 0) Winkel += 2 * PI;   // um ins Intervall [0,360] zu kommen

    Winkel = Winkel * (180/PI);        // Umwandlung von rad in Grad
    
    
    Serial.print(x);
    Serial.print("  ");
    Serial.print(y);
    Serial.print("  ");
    Serial.print(z);
    Serial.print("  ");
    Serial.print(sqrt(x*x + y*y + z*z));
    Serial.print("      ");
    Serial.println(Winkel);
             
    lcd.setCursor(2,0);
    lcd.print("      ");
    lcd.setCursor(11,0);
    lcd.print("     ");
    lcd.setCursor(2,1);
    lcd.print("      ");
    lcd.setCursor(11,1);
    lcd.print("     ");
   
    lcd.setCursor(2,0);
    lcd.print(x,1);
    lcd.setCursor(11,0);
    lcd.print(y,1);
    lcd.setCursor(2,1);
    lcd.print(z,1);
    lcd.setCursor(11,1);
    lcd.print(Winkel,0);
        
    delay(200);
        
   }
